
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    group.h
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    13 Dec 2000
 * Purpose:        Type definitions, prototypes, defines for Group class.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/


#ifndef _GROUP
#define _GROUP

#ifndef _STYLE
#include "style.h"
#endif

#ifndef _WORD
#include "word.h"
#endif

#ifndef _LINE
#include "line.h"
#endif


enum { ALIGN_LEFT=0, ALIGN_RIGHT=1, ALIGN_CENTER=2, ALIGN_JUSTIFY=3 };

/* List Sub-Types
 */
enum { OL_TYPE_ARABIC=0, 
  OL_TYPE_UPPER=1, 
  OL_TYPE_LOWER=2, 
  OL_TYPE_ROMAN_UPPER=3,
  OL_TYPE_ROMAN_LOWER=4,
  UL_TYPE_DISC=5, 
  UL_TYPE_SQUARE=6, 
  UL_TYPE_CIRCLE=7,
};


typedef struct g 
{
	String name;

	/* short max_x; */
	Coordinate x0,y0;

	Dimension min_width; /* the minimum width this group will accept! */
	Dimension max_width; /* this is just x1-x0+1 */

	Dimension max_height;

	/* These variables are being kept here for assigning coordinates to
	 * objs, which is done by designating rectanglular areas for these objs
	 * (words) within in a group area. This is truly a group-centric
	 * activity since a group area will be assigned for an entire 
 	 * document, for each cell within a table, for the spaces within 
	 * a frameset--all of these are best thought of as separate drawing
	 * areas. Coordinates as assigned are absolute however.
	 */
	Coordinate x;
#if 0
	Coordinate y;
#endif
	Coordinate xmax, ymax;
	Coordinate xstart, ystart; 
	/* unsigned short width_this_line; */
	/* unsigned short height_this_line; */
	Dimension width;
	Dimension height;
	Word *first_word; /* for applying correct y coordinates */
	unsigned short at_left; /* true=> we just terminated a line */

	struct g *parent;
	struct g *child;
	struct g *sibling;

	/* Group-level style information */
	Color bgcolor;
	Color border_color;
	unsigned char border_size;

	/* These are needed for later writing out of HTML */
	int beginning_new_line : 1;
	int beginning_new_paragraph : 1;
	int new_paragraph_align : 2;

	/* This is needed for drawing */
	int insert_paragraph_space : 1;

	Style *style_stack;
	Style total_style;
	unsigned char style_has_changed;
	unsigned char style_stored_to_word;

	Line *line_list;
	Line *last_line;
	unsigned long total_lines;

	Line *work_line; /* The line we're putting together now. */

	/* Major list type */
	unsigned list_type : 2;
	unsigned list_indent : 7;

	/* Minor list type */
	unsigned list_subtype : 3;

	/* Integer counter data for the OL list. (XX - Why integer?) */
	long list_value;
	long list_start;
	long list_offset;
} 
Group;


#ifndef _SELECT
#include "select.h"
#endif


extern Group *group_new (DisplayDevice*, Group *,Coordinate,Coordinate,
	Dimension,Dimension);
extern void group_free (Group *);

extern int group_get_x (Group *);
extern int group_get_y (Group *);
extern int group_get_width (Group *);

extern void group_push_style(DisplayDevice*,Group *,Style*);
extern void group_drop_style(Group *);
extern void group_get_total_style (DisplayDevice*, Group *,Style*);
extern int group_get_fontsize (Group*);

extern void group_append_word (DisplayDevice*, Group*,Word*);
extern void group_make_line (Group*);
extern void group_make_spacer_line (Group*, Dimension);
extern void group_end_line_make_line (DisplayDevice*, Group*);
extern void group_end_line (DisplayDevice*, Group*);

extern void group_text_dims (DisplayDevice*,Group*,char*,Dimension*,Dimension*,
	Dimension*);

extern void group_redraw (DisplayDevice*,Group*,Coordinate,Coordinate,Coordinate,Coordinate,Coordinate,Coordinate);
extern void group_write_html (FILE *f, DisplayDevice*,Group*);
extern int group_identify_target (Group*,Coordinate,Coordinate,SelectionPoint*);

extern Group* group_add_child (DisplayDevice*, Group *, Coordinate , Coordinate , Coordinate , Coordinate );

extern int group_geometry (DisplayDevice*, Group*, Word*);
extern void group_dump (FILE*,Group*);

extern void tagarray_init (void);

#endif

