
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    main.c
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    11 Dec 2000
 * Purpose:        main() routine
 *----------------------------------------------------------------------
 * Changes:
 * 28 Nov 2000, zeyr@excite.com: added splash screen, quotation popup
 * 29 Nov 2000, zeyr@excite.com: added another quotation
 * 7 Dec 2000, zeyr@excite.com: fixed scrolling bug, added End key support
 * 11 Dec 2000, zeyr@excite.com: moved interaction() to interact.c
 *--------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#ifdef TURBO_C
#include <conio.h>
#include <alloc.h>
#endif

#include "defs.h"
#include "display.h"
#include "main.h"
#include "parse.h"
#include "word.h"
#include "group.h"
#include "error.h"
#include "svga.h"
#include "malloc.h"
#include "interact.h"


#ifdef TURBO_C
/* Weird Borland kludge to increase the stack size a bit.
 */
extern unsigned _stklen = 40000;
#endif


/* beest
 * by Zach Smith
 * Copyright (C) 2000, All Rights Reserved.
 */

int debug_mode;
int dump_mode;
int vga_mode;
int ps_mode;
int hex_mode;
int noquote_mode;

FILE *logfile; 


/*-----------------------------------------------------------------------*/



int
main(int argc,char **argv)
{
	Word *w;
	FILE *f;
	int i;
	DisplayDevice *dpy;
	int width_we_are_trying;
	Color default_foreground, default_background;
	Color pointer_foreground, pointer_border;
	char *default_family;
	Group *toplevel = NULL;
	time_t twelcome;	
	time_t t0;	
	time_t t;	
	char path [256];

	tagarray_init();

#ifdef TURBO_C
	printf ("Available memory: %lu bytes\n", farcoreleft());
	printf ("Word size: %u\n", sizeof(Word));
	printf ("Line size: %u\n", sizeof(Line));
	printf ("Group size: %u\n", sizeof(Group));
	printf ("Style size: %u\n", sizeof(Style));
	printf ("DisplayDevice size: %u\n", sizeof(DisplayDevice));
#endif

#ifndef TURBO_C
	/* XX 	
	 * Need unique name here in case of multiuser .. e.g. beest.$USER
	 */
	logfile = fopen ("/tmp/beest.log", "w");
#else
	logfile = fopen ("\\beest.log", "w");
#endif
	if (!logfile) {
		/* Current directory might be read-only or full? */
		/* Ignore for now */
	}

	dump_mode=FALSE;
	hex_mode=FALSE;
	debug_mode=FALSE;
	vga_mode=TRUE;
	ps_mode=FALSE;
	noquote_mode=FALSE;

	/* Set up default colors of black text on white background,
	 * black pointer with grey border.
	 */
	default_foreground = 0x0;
	default_background = 0xffffff;
	pointer_foreground = 0x0;
	pointer_border     = 0xa0a0a0;
	default_family = "Times";

	*path=0;

	for (i=1; i<argc; i++) {
		char *arg = argv[i];
		char *next = i<(argc-1) ? argv[i+1] : NULL;
		if (!strcmp("-debug",arg)) 
			debug_mode=TRUE;
		else if (!strcmp("-text",arg)) {
			vga_mode=FALSE;
			ps_mode=FALSE;
		} else if (!strcmp("-vga",arg)) {
			vga_mode=TRUE;
			ps_mode=FALSE;
		} else if (!strcmp("-noquote",arg)) {
			noquote_mode=TRUE;
		} else if (!strcmp("-ps",arg)) {
			ps_mode=TRUE;
			vga_mode=FALSE;
		} else if (!strcmp("-hex",arg)) 
			hex_mode=TRUE;
		else if (!strcmp("-family",arg))  {
			if (next) default_family = next;
			i++;
		}
		else if (!strcmp("-dump",arg)) 
			dump_mode=TRUE;
		else if (!strcmp("-reverse",arg)) {
			int tmp = default_foreground;
			default_foreground = default_background;
			default_background = tmp;
		}
		else if (!*path && *arg != '-') 
			strcpy (path,arg);
		else usage();
	}
	
	if (!*path) usage();

	printf("This is Beest, version %s\n", PROGRAM_VERSION);
	printf("Copyright (C) 2000 by Zach Smith\n");
	printf("This program is provided AS IS, without any warranty whatsoever,\n");
	printf("and is covered by the GNU Public License. For more information, \n");
	printf("see the included file COPYING.\n");
	printf("The author is reachable at zeyr@excite.com.\n");
	printf("Also note:\n");
	printf("Compiled-in fonts (Times, Helvetica, Courier, and NewCenturySchoolbook) are \n");
	printf("Copyright (C) 1984, 1987 Adobe Systems Incorporated. \
All Rights Reserved, and \nCopyright (c) 1988, 1991 Digital Equipment Corporation. All Rights Reserved.\n");
	printf("---\n\n");

	/* Open the input file, trying a variety of extensions.
	 */
	f=fopen (path,"r"); 
	if (!f) { 
		strcat (path, ".html");
		f=fopen (path,"r"); 
		if (!f) {
			path [strlen(path)-5] = 0;
			strcat (path, ".htm");
			f=fopen (path,"r"); 
			if (!f) {
				path [strlen(path)-4] = 0;
				strcat (path, ".HTM");
				f=fopen (path,"r"); 
				if (!f) {
					fprintf(stderr, "cannot open input file\n"); return -1;
				}
			}
		}
	}

	if (!dump_mode) {
		if (!vga_mode) {
			error_handler ("console support removed!");
		} else {
			dpy = svga_init (640,480,
				default_foreground, default_background,
				pointer_foreground, pointer_border,
				default_family); 
		}
		if (!dpy)
			error_handler ("unable to open display");
	}

	/* Display the welcome screen if we are using
	 * a VGA or larger display.
	 */
	if (!dump_mode && dpy->width > 500) {
		Dimension h;

		draw_logo (dpy);

		dpy->set_font_complete ("Helvetica",12,0);
		h = dpy->get_font_height();
		if (!h) error_handler ("SVGA: style set but no font data");
		draw_title_and_copyright (dpy, 125,260,h);
	}

	twelcome = t0 = time(NULL);
	w=word_parse (f);
	fclose(f);
	t = time(NULL);
	if (logfile) fprintf (logfile, "seconds to read file: %lu\n", t-t0);

	if (w) {
		if (dump_mode && logfile) {
			word_dump (logfile, w);
			exit (0);
		}
		else 
		{
			int tries=0;

			width_we_are_trying = dpy->width;
			
			while (tries < 4) {
				int rv;

				t0 = time(NULL);
				toplevel=
				  group_new (dpy, NULL,0,0,width_we_are_trying,MAX_COORD);
				rv = group_geometry (dpy, toplevel, w);

				if (!rv) group_free (toplevel);

				fclose(f);
				t = time(NULL);
				if (logfile)
				 fprintf (logfile, "seconds to process geometry: %lu\n", t-t0);
				if (rv)
					break;

				/* Increase the screen width in an attempt to
				 * accommodate wide content. Horizontal scrolling
				 * to be added later...
				 */
				if (width_we_are_trying < 641) {
					width_we_are_trying = 800;
				} else
				if (width_we_are_trying < 801) {
					width_we_are_trying = 1024;
				} else
				if (width_we_are_trying < 1025) {
					width_we_are_trying = 1280;
				}

				tries++;
			}

			if (tries==8) {
				error_handler("this HTML file requires too wide a screen, or internal error");
			}

			if (logfile)
			  fprintf (logfile, "document geometry was measured as %lu by %lu\n",
				toplevel->width,
				toplevel->height);

#if 0
			/* Make sure the welcome screen stays
			 * visible for at least one second.
			 */
			if (twelcome == time(NULL))
				sleep(1);
#endif

			interaction (dpy, toplevel);
		}
	}
	else
		warning_handler ("parser returned NO data");

	{
		unsigned long alloced, freed;

		my_malloc_total (&alloced, &freed);

		if (logfile) {
		  fprintf (logfile, "Total amount of memory allocated: %lu bytes\n",
			alloced);
#if 0
		  fprintf (logfile, "Total amount of memory freed: %lu bytes\n",
			freed);
#endif
		}
	}

	if (!dump_mode)
		dpy->close();

	if (logfile) fclose (logfile);

	return 0;
}

