
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    special.c
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        Conversion of special expressions to character codes;
 *                 management of hash for that purpose.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/




#include <stdio.h>

#include "defs.h"


typedef struct {
	char *expr;
	unsigned short code;
} HashPair;

static HashPair A[] = {
  {"AElig",0}, {"Aacute",0}, {"Acirc",0}, {"Agrave",0}, {"Aring",0}, {"Atilde",0}, {"Auml",0}, {NULL,0}
};

static HashPair C[] = {
  {"Ccedil",0},  {NULL,0}
};

static HashPair E[] = {
  {"ETH",0}, {"Eacute",0}, {"Ecirc",0}, {"Egrave",0}, {"Euml",0}, {NULL,0}
};

static HashPair I[] = {
  {"Iacute",0}, {"Icirc",0}, {"Igrave",0}, {"Iuml",0}, {NULL,0}
};

static HashPair N[] = {
  {"Ntilde",0},  {NULL,0}
};

static HashPair O[] = {
  {"Oacute",0}, {"Ocirc",0}, {"Ograve",0}, {"Oslash",0}, {"Otilde",0}, {"Ouml",0},  {NULL,0}
};

static HashPair T[] = {
  {"THORN",0},  {NULL,0}
};

static HashPair U[] = {
  {"Uacute",0}, {"Ucirc",0}, {"Ugrave",0}, {"Uuml",0},  {NULL,0}
};

static HashPair Y[] = {
  {"Yacute",0},  {NULL,0}
};

static HashPair a[] = {
  {"amp",'&'}, {"aacute",0}, {"acirc",0}, {"acute",0}, {"aelig",0}, {"agrave",0}, {"aring",0}, {"atilde",0}, {"auml",0}, {NULL,0}
};

static HashPair b[] = {
  {"brvbar",0}, {"bull",0},  {NULL,0}
};

static HashPair c[] = {
  {"ccedil",0}, {"cedil",0}, {"cent",0}, {"copy",0}, {"curren",0},  {NULL,0}
};

static HashPair d[] = {
  {"divide",0},  {NULL,0}
};

static HashPair e[] = {
  {"eacute",0}, {"ecirc",0}, {"egrave",0}, {"eth",0}, {"euml",0},  {NULL,0}
};

static HashPair f[] = {
  {"frac12",0}, {"frac14",0}, {"frac34",0},  {NULL,0}
};

static HashPair i[] = {
  {"iacute",0}, {"icirc",0}, {"iexcl",0}, {"igrave",0}, {"iquest",0}, {"iuml",0},  {NULL,0}
};

static HashPair g[] = {
  {"gt",'>'}, {NULL,0}
};

static HashPair l[] = {
  {"lt",'<'}, {"laquo",0}, {"ldquo",0}, {"ldqyo",0}, {"lquo",0}, {"lsquo",0},  {NULL,0}
};

static HashPair m[] = {
  {"macr",0}, {"mdash",0}, {"micro",0}, {"middot",0},  {NULL,0}
};

static HashPair n[] = {
  {"nbsp",0}, {"ndash",0}, {"not",0}, {"ntilde",0},  {NULL,0}
};

static HashPair o[] = {
  {"oacute",0}, {"ocirc",0}, {"ograve",0}, {"ordf",0}, {"ordm",0}, {"oslash",0}, {"otilde",0}, {"ouml",0},  {NULL,0}
};

static HashPair p[] = {
  {"para",0}, {"plus",0}, {"plusmn",0}, {"pound",0},  {NULL,0}
};

static HashPair r[] = {
  {"raquo",0}, {"rdquo",0}, {"reg",0}, {"rquo",0}, {"rsquo",0},  {NULL,0}
};

static HashPair s[] = {
  {"sect",0}, {"shy",0}, {"sup1",0}, {"sup2",0}, {"sup3",0}, {"szlig",0}, {NULL,0}
};

static HashPair t[] = {
  {"thorn",0}, {"times",0},  {NULL,0}
};

static HashPair u[] = {
  {"uacute",0}, {"ucirc",0}, {"ugrave",0}, {"uml",0}, {"uuml",0}, {NULL,0}
};

static HashPair y[] = {
  {"yacute",0}, {"yen",0}, {"yuml",0}, {NULL,0}
};

static HashPair *hash_lower[] = {
  a, b, c, d, e, f, g, NULL, i, NULL, NULL, l, m, n, o, p, NULL,
  r, s, t, u, NULL, NULL, NULL, y, NULL,
};

static HashPair *hash_upper[] = {
  A, NULL, C, NULL, E, NULL, NULL, NULL, I, NULL, NULL, NULL, NULL, N, O, NULL,
  NULL, NULL, NULL, T, U, NULL, NULL, NULL, Y, NULL,
};


int
special_valid_first_char (int ch) {
	if (ch < 'A' || ch > 'z')
		return FALSE;
	if (ch > 'Z' && ch < 'a')
		return FALSE;
	if (ch < 'a') 
		return (hash_upper[ch-'A']) ? TRUE : FALSE;
	else
		return (hash_lower[ch-'a']) ? TRUE : FALSE;
	
}

int
special_translate (char *s) {
	HashPair *h;
	int ix=0;
	char *s2;

/* printf ("trying to translate '%s'... ", s); */
	if (!special_valid_first_char (*s))
		return 0;

	if (*s < 'a') 
		h = hash_lower[*s-'A'];
	else
		h = hash_lower[*s-'a'];

	while ((s2 = h[ix].expr)) {
		if (!strcmp (s,s2)) {
			int rv = h[ix].code;
/* printf ("returning %d\n", rv); */
			return rv;
		}
		ix++;
	}
/* printf ("failed.\n"); */
	return 0;
}



