
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    string.c
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        String manipulation.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/


#include <stdio.h>
#include <ctype.h>

#include "defs.h"
#include "main.h"
#include "error.h"



int
my_case_insensitive_strcmp (String s1, String s2)
{
	CHECK_PARAM_NOT_NULL(s1);
	CHECK_PARAM_NOT_NULL(s2);

	while (1) {
		if (tolower(*s1) != tolower(*s2))
			return FALSE;
		s1++;
		s2++;

		if (!*s1 && !*s2)
			return TRUE;
		if (!*s1 || !*s2)
			break;
	}

	return FALSE;
}


/* Convert two-byte compressed value to a 12-bit signed number
 */
int
c2toi_signed (String s) {
	short tmp, tmp2;
	short value=0;

	tmp=*s++;
	if (tmp==32) tmp2=0;
	else if (tmp=='.') tmp2=1;
	else if (tmp>='0' && tmp<='9') tmp2=tmp-'0'+2;
	else if (tmp>='a' && tmp<='z') tmp2=tmp-'a'+2+10;
	else if (tmp>='A' && tmp<='Z') tmp2=tmp-'A'+2+10+26;
	value=tmp2;

	tmp=*s;
	if (tmp==32) tmp2=0;
	else if (tmp=='.') tmp2=1;
	else if (tmp>='0' && tmp<='9') tmp2=tmp-'0'+2;
	else if (tmp>='a' && tmp<='z') tmp2=tmp-'a'+2+10;
	else if (tmp>='A' && tmp<='Z') tmp2=tmp-'A'+2+10+26;
	value <<= 6;
	value+=tmp2;

	if (value>2047) value -= 4096;
	return value;
}


int
h2toi (String s) {
	int tmp;
	int ch;
	tmp = tolower(*s++);
	if (tmp>'9') tmp-=('a'-10); 
	else tmp-='0';
	ch=16*tmp;
	tmp = tolower(*s++);
	if (tmp>'9') tmp-=('a'-10); 
	else tmp-='0';
	ch+=tmp;
	return ch;
}

int
h1toi (String s) {
	int tmp;
	int ch;
	tmp = tolower(*s);
	if (tmp>'9') tmp-=('a'-10); 
	else tmp-='0';
	ch=tmp;
	return ch;
}


/* Routine to generate old-fashioned (that is to say, Roman) 
 * Roman numerals.
 */
void
generate_roman_numeral (unsigned int num, String str, int len, int uppercase) 
{
	int i,x,v;
	int index=0;
	unsigned char ch_i, ch_x, ch_v;

	CHECK_PARAM_NOT_NULL(str);
	if (!len) return;

	if (uppercase) {
		ch_i=(Char)"I";
		ch_x=(Char)"X";
		ch_v=(Char)"V";
	} else {
		ch_i=(Char)"i";
		ch_x=(Char)"x";
		ch_v=(Char)"v";
	}

	len--;
	str[len]=0;

	if (num>25) num=25;
	else if (num<1) num=1;

	x=num/10;
	num -= x*10;

	v=num/5;
	num -= v*5;

	i = num;

	while (x--) {
		str[index++]=ch_x; if (index==len) return;
	}

	if (v) {
		str[index++]=ch_v; if (index==len) return;
	}

	while (i--) {
		str[index++]=ch_i; if (index==len) return;
	}

	str[index++]=0;
}


void
itoc2_signed (int x, String str)
{
	int x1,x2;
	long y;

	y=x;
	if (y<0) y+=4096;
	x1 = y/64;
	x2 = y&63;

	if (x1==0) *str=' ';
	else
	if (x1==1) *str='.';
	else
	if (x1<(10+2)) *str = x1+'0'-2;
	else
	if (x1<(26+10+2)) *str = x1+'a'-(10+2);
	else
	*str = x1+'A'-(26+10+2);

	str++;

	if (x2==0) *str=' ';
	else
	if (x2==1) *str='.';
	else
	if (x2<(10+2)) *str = x2+'0'-2;
	else
	if (x2<(26+10+2)) *str = x2+'a'-(10+2);
	else
	*str = x2+'A'-(26+10+2);

}

